use serde::Serialize;

use super::ast_node::WasmAstNode;
use super::macros::to_wasm;
use super::standard_properties::WasmStandardProperties;
use super::to_wasm::ToWasm;
use super::AdditionalProperties;
#[cfg(feature = "wasm_test")]
use crate::compare::ElispFact;
use crate::types::Section;
use crate::wasm::to_wasm::ToWasmStandardProperties;

#[derive(Debug, Serialize)]
#[serde(tag = "ast_node")]
#[serde(rename = "section")]
pub struct WasmSection<'s, 'p> {
    pub(crate) standard_properties: WasmStandardProperties,
    #[serde(flatten)]
    pub(crate) additional_properties: AdditionalProperties<'s, 'p>,
    pub(crate) children: Vec<WasmAstNode<'s, 'p>>,
}

to_wasm!(
    WasmSection<'s, 'p>,
    Section<'s>,
    original,
    wasm_context,
    standard_properties,
    {
        let children = original
            .children
            .iter()
            .map(|child| {
                child
                    .to_wasm(wasm_context.clone())
                    .map(Into::<WasmAstNode<'_, '_>>::into)
            })
            .collect::<Result<Vec<_>, _>>()?;

        Ok(WasmSection {
            standard_properties,
            additional_properties: AdditionalProperties::default(),
            children,
        })
    }
);

impl<'s, 'p> Into<WasmAstNode<'s, 'p>> for WasmSection<'s, 'p> {
    fn into(self) -> WasmAstNode<'s, 'p> {
        WasmAstNode::Section(self)
    }
}

#[cfg(feature = "wasm_test")]
impl<'s, 'p> ElispFact<'s> for WasmSection<'s, 'p> {
    fn get_elisp_name<'b>(&'b self) -> std::borrow::Cow<'s, str> {
        "section".into()
    }
}
